package com.jeeplus.business.tender.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jeeplus.business.finance.util.EasyPoiUtil;
import com.jeeplus.business.reimbursement.service.dto.JyReimbursementInfoDTO;
import com.jeeplus.business.tender.service.DTO.TenderDTO;
import com.jeeplus.business.tender.service.TenderService;
import com.jeeplus.business.useSeal.service.dto.SignetDTO;
import com.jeeplus.common.excel.ExcelOptions;
import com.jeeplus.common.excel.annotation.ExportMode;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.logging.annotation.ApiLog;
import com.jeeplus.logging.constant.enums.LogTypeEnum;
import com.jeeplus.pubmodules.purchase.service.dto.MaterialInfoDto;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@RestController
@Api(tags ="招投标")
@RequestMapping(value = "/tender")
public class TenderController {

    @Resource
    private TenderService tenderService;





    /**
     * 投标列表
     */
    @ApiOperation(value = "投标列表查询")
    @GetMapping("/list")
    public ResponseEntity<IPage<TenderDTO>> list(Page<TenderDTO> page, TenderDTO dto) throws Exception{
        IPage<TenderDTO> iPage = tenderService.list(page, dto);

        return ResponseEntity.ok(iPage);
    }

    /**
     * 新增/修改
     * @param dto
     * @return
     */
    @ApiOperation(value = "新增、修改")
    @PostMapping("/save")
    public ResponseEntity<String> save(@RequestBody TenderDTO dto) throws Exception{
        String s = tenderService.save(dto);
        return ResponseEntity.ok(s);
    }

    /**
     * 查询
     * @param id
     * @return
     */
    @ApiOperation(value = "查询")
    @GetMapping("/findById")
    public ResponseEntity<TenderDTO> findById(@RequestParam String id) throws Exception{
        TenderDTO dto = tenderService.findById(id);
        return ResponseEntity.ok(dto);
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @GetMapping("/remove")
    public ResponseEntity<String> remove(@RequestParam String id){
        String s = tenderService.remove(id);
        return ResponseEntity.ok(s);
    }

    /**
     * 抄送
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @GetMapping("/copyToLeader")
    public ResponseEntity<String> copyToLeader(@RequestParam String id){
        String s = tenderService.copyToLeader(id);
        return ResponseEntity.ok(s);
    }

    @ApiLog(value = "导出用印数据", type = LogTypeEnum.EXPORT)
    @GetMapping("export")
    @ApiOperation(value = "导出用印数据")
    public void exportFile(Page<TenderDTO> page, TenderDTO dto, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = options.getFilename();
        String sheetName = options.getSheetName();
        List<TenderDTO> result = new ArrayList<>();
        if ( ExportMode.current.equals ( options.getMode() ) ) {
            result = tenderService.findList(page,dto).getRecords();
        } else if (ExportMode.selected.equals ( options.getMode() )) {
            result = tenderService.findList(page,dto).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            result = tenderService.findList(page,dto).getRecords();
        }
        EasyPoiUtil.exportExcel ( result, sheetName,  sheetName, TenderDTO.class, fileName, response );

    }











}
